/*
 * nokia_5110.h
 *
 * Author : Andrei Cain
 */
#include "english_6x8_pixel.h"
#define BIT(i)  (1<<i)  

#define LCD_RST_set	PORTC|=BIT(2)	//RST
#define LCD_RST_clr	PORTC&=~BIT(2)

#define LCD_CE_set	PORTC|=BIT(3)	//CE
#define LCD_CE_clr	PORTC&=~BIT(3)

#define LCD_DC_set	PORTC|=BIT(4)	//DC
#define LCD_DC_clr	PORTC&=~BIT(4)

#define SDIN_set	PORTC|=BIT(5)	//DIN   
#define SDIN_clr	PORTC&=~BIT(5)

#define SCLK_set	PORTC|=BIT(6)	//CLK
#define SCLK_clr	PORTC&=~BIT(6)

/*-----------------------------------------------------------------------
LCD_write_byte	:	write data to LCD

input parameters:	dat		data
					command	command

-----------------------------------------------------------------------*/
void LCD_write_byte(unsigned char dat, unsigned char command) {
	unsigned char i;
	LCD_CE_clr;
	if (command == 0)
		LCD_DC_clr;
	else
		LCD_DC_set;
	for (i=0; i<8; i++) {
		if (dat&0x80)
			SDIN_set;
		else
			SDIN_clr;
		SCLK_clr;
		dat = dat << 1;
		SCLK_set;
	}
	LCD_CE_set;
}

void delay_1us(void) {	//delay 1us
	_delay_us(1);
}

void delay_1ms(void) {	// delay 1ms
	_delay_ms(1);
}
  
  
void LCD_clear(void) {	// LCD clearance
	unsigned int i;
	LCD_write_byte(0x0c, 0);
	LCD_write_byte(0x80, 0);
	for (i=0; i<504; i++)
		LCD_write_byte(0, 1);
}
  

void LCD_init(void) {	// LCD initialization
	DDRC |= 0xFF;    

	LCD_RST_clr;
	delay_1us();
	LCD_RST_set;

	LCD_CE_clr;
	delay_1us();
	LCD_CE_set;

	delay_1us();

	LCD_write_byte(0x21, 0);	// set LCD mode
	LCD_write_byte(0xc8, 0);	// set bias voltage
	LCD_write_byte(0x06, 0);	// temperature correction
	LCD_write_byte(0x13, 0);	// 1:48
	LCD_write_byte(0x20, 0);	// use bias command
	LCD_write_byte(0x0c, 0);	// set LCD mode,display normally
    
	LCD_CE_clr;    
}

/*-----------------------------------------------------------------------
LCD_set_XY	:	Set the LCD coordinate functions

input parameters:	X	0-83
					Y	0-5

-----------------------------------------------------------------------*/
void LCD_set_XY(unsigned char X, unsigned char Y) {
	LCD_write_byte(0x40 | Y, 0);	// column
	LCD_write_byte(0x80 | X, 0);	// row
}

/*-----------------------------------------------------------------------
LCD_write_char	:	Display English characters

input parameters:	c	char to display

-----------------------------------------------------------------------*/
void LCD_write_char(unsigned char c) {
	unsigned char line;
	c -= 32;
	for (line=0; line<6; line++)
		LCD_write_byte(font6x8[c][line], 1);
}

/*-----------------------------------------------------------------------
LCD_write_english_string	:	Display English strings

input parameters:	X	0-83
					Y	0-5
					*s	string

-----------------------------------------------------------------------*/
void LCD_write_english_string(unsigned char X, unsigned char Y, char *s) {
	LCD_set_XY(X, Y);
	while (*s) {
		LCD_write_char(*s);
		s++;
	}
}

/*-----------------------------------------------------------------------
LCD_write_int	:	Display integer

input parameters:	X	0-83
					Y	0-5
					num	integer

-----------------------------------------------------------------------*/
void LCD_write_int(unsigned char X, unsigned char Y, int num) {
	unsigned char line;
	LCD_set_XY(X, Y);
	for (line=0; line<6; line++)
		LCD_write_byte(font6x8[num+16][line], 1);
}

/*-----------------------------------------------------------------------
LCD_draw_map	:	Bitmap drawing function

input parameters:	X, Y	starting point X, Y
					*map	bitmap data
					Pix_x	height
					Pix_y	width

-----------------------------------------------------------------------*/
void LCD_draw_bmp_pixel(unsigned char X, unsigned char Y, unsigned char *map,
						unsigned char Pix_x, unsigned char Pix_y) {
	unsigned int i, n;
	unsigned char row;
	if (Pix_y % 8 == 0)
		row = Pix_y / 8;	//calculate how many lines are needed
	else
		row = Pix_y / 8+1;
	for (n=0; n<row; n++) {
		LCD_set_XY(X, Y);
		for (i=0; i<Pix_x; i++)
			LCD_write_byte(map[i+n*Pix_x], 1);
		Y++;		//change line
	}
}